function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component, Children } from 'react';
import PropTypes from 'prop-types';
import { storeShape } from '../utils/PropTypes';
import warning from '../utils/warning';

import { ReactReduxContext } from "./context";

var didWarnAboutReceivingStore = false;
function warnAboutReceivingStore() {
  if (didWarnAboutReceivingStore) {
    return;
  }
  didWarnAboutReceivingStore = true;

  warning('<Provider> does not support changing `store` on the fly. ' + 'It is most likely that you see this error because you updated to ' + 'Redux 2.x and React Redux 2.x which no longer hot reload reducers ' + 'automatically. See https://github.com/reduxjs/react-redux/releases/' + 'tag/v2.0.0 for the migration instructions.');
}

export function createProvider() {
  var Provider = function (_Component) {
    _inherits(Provider, _Component);

    function Provider(props) {
      _classCallCheck(this, Provider);

      var _this = _possibleConstructorReturn(this, _Component.call(this, props));

      var store = props.store;


      _this.state = {
        storeState: store.getState(),
        store: store
      };
      return _this;
    }

    Provider.prototype.componentDidMount = function componentDidMount() {
      this._isMounted = true;
      this.subscribe();
    };

    Provider.prototype.componentWillUnmount = function componentWillUnmount() {
      if (this.unsubscribe) this.unsubscribe();

      this._isMounted = false;
    };

    Provider.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
      if (this.props.store !== prevProps.store) {
        if (this.unsubscribe) this.unsubscribe();

        this.subscribe();
      }
    };

    Provider.prototype.subscribe = function subscribe() {
      var _this2 = this;

      var store = this.props.store;


      this.unsubscribe = store.subscribe(function () {
        var newStoreState = store.getState();

        if (!_this2._isMounted) {
          return;
        }

        _this2.setState(function (providerState) {
          // If the value is the same, skip the unnecessary state update.
          if (providerState.storeState === newStoreState) {
            return null;
          }

          return { storeState: newStoreState };
        });
      });

      // Actions might have been dispatched between render and mount - handle those
      var postMountStoreState = store.getState();
      if (postMountStoreState !== this.state.storeState) {
        this.setState({ storeState: postMountStoreState });
      }
    };

    Provider.prototype.render = function render() {
      var ContextProvider = this.props.contextProvider || ReactReduxContext.Provider;

      return React.createElement(
        ContextProvider,
        { value: this.state },
        this.props.children
      );
    };

    return Provider;
  }(Component);

  Provider.propTypes = {
    store: storeShape.isRequired,
    children: PropTypes.element.isRequired,
    contextProvider: PropTypes.object
  };

  return Provider;
}

export default createProvider();