var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import hoistStatics from 'hoist-non-react-statics';
import invariant from 'invariant';
import PropTypes from 'prop-types';
import React, { Component, PureComponent } from 'react';
import { isValidElementType } from 'react-is';

import { ReactReduxContext } from "./context";
import { storeShape } from "../utils/PropTypes";

var hotReloadingVersion = 0;

export default function connectAdvanced(
/*
  selectorFactory is a func that is responsible for returning the selector function used to
  compute new props from state, props, and dispatch. For example:
     export default connectAdvanced((dispatch, options) => (state, props) => ({
      thing: state.things[props.thingId],
      saveThing: fields => dispatch(actionCreators.saveThing(props.thingId, fields)),
    }))(YourComponent)
   Access to dispatch is provided to the factory so selectorFactories can bind actionCreators
  outside of their selector as an optimization. Options passed to connectAdvanced are passed to
  the selectorFactory, along with displayName and WrappedComponent, as the second argument.
   Note that selectorFactory is responsible for all caching/memoization of inbound and outbound
  props. Do not use connectAdvanced directly without memoizing results between calls to your
  selector, otherwise the Connect component will re-render on every state or props change.
*/
selectorFactory) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$getDisplayName = _ref.getDisplayName,
      getDisplayName = _ref$getDisplayName === undefined ? function (name) {
    return 'ConnectAdvanced(' + name + ')';
  } : _ref$getDisplayName,
      _ref$methodName = _ref.methodName,
      methodName = _ref$methodName === undefined ? 'connectAdvanced' : _ref$methodName,
      _ref$shouldHandleStat = _ref.shouldHandleStateChanges,
      shouldHandleStateChanges = _ref$shouldHandleStat === undefined ? true : _ref$shouldHandleStat,
      _ref$consumer = _ref.consumer,
      consumer = _ref$consumer === undefined ? ReactReduxContext.Consumer : _ref$consumer,
      _ref$storeKey = _ref.storeKey,
      storeKey = _ref$storeKey === undefined ? 'store' : _ref$storeKey,
      _ref$withRef = _ref.withRef,
      withRef = _ref$withRef === undefined ? false : _ref$withRef,
      _ref$forwardRef = _ref.forwardRef,
      forwardRef = _ref$forwardRef === undefined ? false : _ref$forwardRef,
      connectOptions = _objectWithoutProperties(_ref, ['getDisplayName', 'methodName', 'shouldHandleStateChanges', 'consumer', 'storeKey', 'withRef', 'forwardRef']);

  invariant(!withRef, "withRef is removed. To access the wrapped instance, use a ref on the connected component");

  invariant(storeKey === 'store', 'storeKey has been removed. To use a custom redux store for a single component, ' + 'create a custom React context with React.createContext() and pass the Provider to react-redux\'s provider ' + 'and the Consumer to this component as in <Provider context={context.Provider}><' + 'ConnectedComponent consumer={context.Consumer} /></Provider>');

  var version = hotReloadingVersion++;

  return function wrapWithConnect(WrappedComponent) {
    invariant(isValidElementType(WrappedComponent), 'You must pass a component to the function returned by ' + (methodName + '. Instead received ' + JSON.stringify(WrappedComponent)));

    var wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';

    var displayName = getDisplayName(wrappedComponentName);

    var selectorFactoryOptions = _extends({}, connectOptions, {
      getDisplayName: getDisplayName,
      methodName: methodName,
      shouldHandleStateChanges: shouldHandleStateChanges,
      displayName: displayName,
      wrappedComponentName: wrappedComponentName,
      WrappedComponent: WrappedComponent
    });

    var OuterBaseComponent = connectOptions.pure ? PureComponent : Component;

    function createChildSelector(store) {
      return selectorFactory(store.dispatch, selectorFactoryOptions);
    }

    var ConnectInner = function (_Component) {
      _inherits(ConnectInner, _Component);

      function ConnectInner(props) {
        _classCallCheck(this, ConnectInner);

        var _this = _possibleConstructorReturn(this, _Component.call(this, props));

        _this.state = {
          wrapperProps: props.wrapperProps,
          store: props.store,
          error: null,
          childPropsSelector: createChildSelector(props.store),
          childProps: {}
        };

        _this.state = _extends({}, _this.state, ConnectInner.getChildPropsState(props, _this.state));
        return _this;
      }

      ConnectInner.getChildPropsState = function getChildPropsState(props, state) {
        try {
          var childPropsSelector = state.childPropsSelector;


          if (props.store !== state.store) {
            childPropsSelector = createChildSelector(props.store);
          }

          var nextProps = childPropsSelector(props.storeState, props.wrapperProps);
          if (nextProps === state.childProps) return null;

          return { childProps: nextProps, store: props.store, childPropsSelector: childPropsSelector };
        } catch (error) {
          return { error: error };
        }
      };

      ConnectInner.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
        var nextChildProps = ConnectInner.getChildPropsState(props, state);

        if (nextChildProps === null) {
          return null;
        }

        return _extends({}, nextChildProps, {
          wrapperProps: props.wrapperProps
        });
      };

      ConnectInner.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
        var childPropsChanged = nextState.childProps !== this.state.childProps;
        var hasError = !!nextState.error;

        return childPropsChanged || hasError;
      };

      ConnectInner.prototype.render = function render() {
        if (this.state.error) {
          throw this.state.error;
        }

        return React.createElement(WrappedComponent, _extends({}, this.state.childProps, { ref: this.props.forwardRef }));
      };

      return ConnectInner;
    }(Component);

    ConnectInner.propTypes = {
      wrapperProps: PropTypes.object,
      store: storeShape
    };

    function createWrapperPropsMemoizer() {
      var result = void 0,
          prevProps = void 0;

      return function wrapperPropsMemoizer(props) {
        if (props === prevProps) {
          return result;
        }

        var contextConsumer = props.contextConsumer,
            forwardRef = props.forwardRef,
            wrapperProps = _objectWithoutProperties(props, ['contextConsumer', 'forwardRef']);

        result = { contextConsumer: contextConsumer, forwardRef: forwardRef, wrapperProps: wrapperProps };

        return result;
      };
    }

    var Connect = function (_OuterBaseComponent) {
      _inherits(Connect, _OuterBaseComponent);

      function Connect(props) {
        _classCallCheck(this, Connect);

        var _this2 = _possibleConstructorReturn(this, _OuterBaseComponent.call(this, props));

        _this2.version = version;

        _this2.renderInner = _this2.renderInner.bind(_this2);

        _this2.wrapperPropsMemoizer = createWrapperPropsMemoizer();
        return _this2;
      }

      Connect.prototype.renderInner = function renderInner(providerValue) {
        var storeState = providerValue.storeState,
            store = providerValue.store;

        var _wrapperPropsMemoizer = this.wrapperPropsMemoizer(this.props),
            forwardRef = _wrapperPropsMemoizer.forwardRef,
            wrapperProps = _wrapperPropsMemoizer.wrapperProps;

        return React.createElement(ConnectInner, {
          key: this.version,
          storeState: storeState,
          store: store,
          wrapperProps: wrapperProps,
          forwardRef: forwardRef
        });
      };

      Connect.prototype.render = function render() {
        var ContextConsumer = this.props.contextConsumer || consumer;

        return React.createElement(
          ContextConsumer,
          null,
          this.renderInner
        );
      };

      return Connect;
    }(OuterBaseComponent);

    Connect.WrappedComponent = WrappedComponent;
    Connect.displayName = displayName;
    Connect.propTypes = {
      contextConsumer: PropTypes.object,
      forwardRef: PropTypes.oneOfType([PropTypes.func, PropTypes.object])

      // TODO We're losing the ability to add a store as a prop. Not sure there's anything we can do about that.


    };var wrapperComponent = Connect;

    if (forwardRef) {
      var forwarded = React.forwardRef(function (props, ref) {
        return React.createElement(Connect, _extends({}, props, { forwardRef: ref }));
      });

      wrapperComponent = forwarded;
    }

    return hoistStatics(wrapperComponent, WrappedComponent);
  };
}